/* -*- Mode: C; c-file-style: "stroustrup" -*- */

/**********************************************
 * SEE COPYING FILE FOR COPYRIGHT INFORMATION *
 **********************************************/

#include <glib.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>


#include "standard.h"
#include "words.h"
#include "corpus.h"
#include "isolatin.h"
#include "config.h"

#define MAXBUF 500
#define STEP 100

gboolean quiet;

char *my_lowercase(char *sen) {
    char *ptr = sen;
    while(*ptr) {
	*ptr = tolower(*ptr);
	ptr++;
    }
    return sen;
}

int AddSentence(char **sen, unsigned long len,
		WordList* wl, Corpus *Corpus)
{
    unsigned long i, nr = 0;
    
    /* Add each sentence word */
    for (i = 0; i < len; i++) {
	int flag; 		/* 1: lowercase; 2: Capital; 3: UPPERCASE */

	if (isCapital(*sen)) flag = 2;
	else if (isUPPERCASE(*sen)) flag = 3;
	else flag = 1;

	nr = word_list_add_word(wl, my_lowercase(*sen));
	sen++;
	
	if (nr) {
	    if (corpus_add_word(Corpus, nr, flag)) return 1;
	} else return 2;
    }
    /* If 'nr' is set, the sentence is not empty. */
    if (nr) {
	/* Add sentence delimiter (0) */
	if (corpus_add_word(Corpus, 0, 1)) return 3;
    } else return 4;
    return 0;
}

int AnalyseCorpus(Corpus *C1, WordList* wl1, char *text1, 
		  Corpus *C2, WordList* wl2, char *text2,
		  guint32 *Nw1, guint32 *Nw2, guint32 *Nsen,
		  guint32 *TotNw1, guint32 *TotNw2, guint32 *TotNsen)
{
    unsigned long len1, len2;
    char *sen1[MAXBUF], *sen2[MAXBUF];
    
    if (!quiet) {
	fprintf(stderr, "\n Sentences\tWords cp1\tWords cp2\n");
	fprintf(stderr," ");
    }
    do {
	/* get a sentence for each corpus (array of words) */
	len1 = NextTextSentence(sen1, &text1, MAXBUF, SOFTDELIMITER, HARDDELIMITER);
	len2 = NextTextSentence(sen2, &text2, MAXBUF, SOFTDELIMITER, HARDDELIMITER);

	if (len1 && len2) {
	    (*TotNsen)++;
	    (*TotNw1) += len1;
	    (*TotNw2) += len2;
	    
	    if (*TotNsen % STEP == 0 && !quiet)
		fprintf(stderr, "\r  %7d\t %7d\t %7d", *TotNsen, *TotNw1, *TotNw2);
	    
	    if (max(len1, len2) <= MAXBUF) {
		(*Nsen)++;
		(*Nw1) += len1;
		(*Nw2) += len2;
		if (AddSentence(sen1, len1, wl1, C1)) return 1;
		if (AddSentence(sen2, len2, wl2, C2)) return 1;
	    }
	}
    } while (text1 != NULL && text2 != NULL);

    if (!quiet)
	fprintf(stderr, "\r  %7d\t %7d\t %7d\n\n", *TotNsen, *TotNw1, *TotNw2);

    if (text1 != NULL || text2 != NULL) return 2;
    else  return 0;
}

int main(int argc, char **argv)
{
    WordList *wordLst1, *wordLst2;

    Corpus *Corpus1, *Corpus2;

    char *text1, *text2;

    gboolean verbose = FALSE;

    guint32 Nw1, Nw2, Nsen;
    guint32 TotNw1, TotNw2, TotNsen;
    guint32 UNw1, UNw2, UNsen;
    guint32 TotUNw1, TotUNw2;
    int result;

    extern char *optarg;
    extern int optind;
    int c;

    quiet = FALSE;
    optind = 0;

    while ((c = getopt(argc, argv, "vqV")) != EOF) {
	switch (c) {
	case 'V':
	    printf(PACKAGE " version " VERSION "\n");
	    exit(0);
	case 'v':
	    verbose = TRUE;
	    break;
	case 'q':
	    quiet = TRUE;
	    break;
	default:
	    fprintf(stderr, "Unknown option -%c\n", c);
	    exit(1);
	}
    }

    if (quiet && verbose) {
	fprintf(stderr, "Quiet and verbose can't work together\n");
	return 1;
    }

    if (argc != 6 + optind) {
	printf("%s: wrong number of arguments\n", argv[0]);
	printf("\tUsage: nat-pre cp1 cp2 lex1 lex2 crp1 crp2\n");
	exit(1);
    }

    if (verbose) {
	printf("\nMaximum sentence length: %d words\n", MAXLEN); 
	printf("Delimiters:     Sentence: '%c'      Paragraph: '%c'\n", SOFTDELIMITER, HARDDELIMITER);
    }

    wordLst1 = word_list_load(argv[optind + 2], NULL);
    if (!wordLst1) {
	if (!quiet)
	    fprintf(stderr, "Error loading words from %s. Creating new list\n", argv[optind + 2]);
	wordLst1 = word_list_new();
	if (!wordLst1) report_error("Not enough memory!");
    }

    wordLst2 = word_list_load(argv[optind + 3], NULL);
    if (!wordLst2) {
	if (!quiet)
	    fprintf(stderr, "Error loading words from %s. Creating new list\n", argv[optind + 3]);
	wordLst2 = word_list_new();
	if (!wordLst2) report_error("Not enough memory!");
    }

    Corpus1 = corpus_new();
    Corpus2 = corpus_new();
    
    TotNw1 = 0;
    TotUNw1 = 0;

    TotNw2 = 0;
    TotUNw2 = 0;

    TotNsen = 0; 

    if (verbose) printf("\nReading %s...\n", argv[optind]);
    text1 = ReadText(argv[optind]);

    if (verbose) printf("Reading %s...\n", argv[optind + 1]);
    text2 = ReadText(argv[optind + 1]);

    if (text1 == NULL || text2 == NULL)
	report_error("ReadText");

    Nw1 = 0; UNw1 = 0;
    Nw2 = 0; UNw2 = 0;

    Nsen = 0; UNsen = 0;
    
    result = AnalyseCorpus(Corpus1, wordLst1, text1,
			   Corpus2, wordLst2, text2,
			   &UNw1, &UNw2, &UNsen, &Nw1, &Nw2, &Nsen);

    TotUNw1 += UNw1;
    TotUNw2 += UNw2;

    TotNw1 += Nw1;
    TotNw2 += Nw2;

    TotNsen += Nsen;

    if (result) {
	if (result == 2) report_error("AnalyseCorpus: lengths do not match");
	else report_error("AnalyseCorpus");
    }

    if (verbose) {
	printf("\nTotal corpus 1:\n");
	printf(" %d words\n", (int) TotNw1);
	printf(" %d words used\n", (int) TotUNw1);
	printf(" %d different words\n",word_list_size(wordLst1));

	printf("\nTotal corpus 2:\n");
	printf(" %d words\n", (int) TotNw2);
	printf(" %d words used\n", (int) TotUNw2);
	printf(" %d different words\n",word_list_size(wordLst2));
    }

    if (verbose) { printf ("Checking corpus sizes\n"); }
    Nw1 = corpus_sentences_nr(Corpus1);
    Nw2 = corpus_sentences_nr(Corpus2);
    if (Nw1 != Nw2)
	report_error("Corpus lengths didn't pass final test");

    if (Nw1 == 0) {
	fprintf(stderr,"No sentences found\n");
	return 1;
    }

    if (verbose) { printf ("Saving lexicon files\n"); }
    if (word_list_save(wordLst1, argv[optind + 2]) != TRUE) report_error("SaveWords 1");
    if (word_list_save(wordLst2, argv[optind + 3]) != TRUE) report_error("SaveWords 2");

    if (verbose) { printf ("Saving corpora files\n"); }
    if (corpus_save(Corpus1, argv[optind + 4])) report_error("SaveCorpus 1");
    if (corpus_save(Corpus2, argv[optind + 5])) report_error("SaveCorpus 2");

    
    word_list_free(wordLst1);
    word_list_free(wordLst2);

    free(text1);
    free(text2);

    corpus_free(Corpus1);
    corpus_free(Corpus2);
    
    return 0;
}

